<?php

/**
 * Micropayment GmbH
 *
 * @link       http://www.micropayment.de
 * @copyright  Copyright (c) 2020, Micropayment GmbH
 * @author     Tobias Eilers
 * @package    Shopware
 * @subpackage Plugins
 */
class Shopware_Plugins_Frontend_MicropaymentOF_Bootstrap extends Shopware_Components_Plugin_Bootstrap
{
  var $version = "1.0.0";
  var $McPay;

  /**
   * getInfo
   *
   * @return array
   */
  public function getInfo()
  {
    $imgPath = dirname(__FILE__).'/img/micropayment-logo.png';
    $img     = 'data:image/png;base64,'.base64_encode(file_get_contents($imgPath));
    $img     = '//www.micropayment.de/resources/?what=img&group=pcidss&show=type-e.2';
    return array(
      'version'     => $this->version,
      'autor'       => 'Micropayment GmbH',
      'label'       => "Micropayment Zahlungsarten Order First",
      'source'      => "Default",
      'description' => '<p><img src="'.$img.'" /></p>'
        .'<p style="font-size:12px; font-weight: bold;">Micropayment GmbH<br>'
        .'<a href="http://r91.micropayment.de" target="_blank"">Jetzt hier registrieren und loslegen.</a></p>'
        .'<iframe src="../MicropaymentOF/CheckUpdate" style="height: 400px"></iframe>',
      'license'     => 'commercial',
      'copyright'   => 'Copyright © 2021, Micropayment GmbH',
      'support'     => 'tobias.eilers@micropayment.de',
      'link'        => 'http://www.micropayment.de/',
    );
  }

  /**
   * getVersion
   *
   * @return string|null
   */
  public function getVersion()
  {
    return $this->version;
  }

  /**
   * getLabel
   *
   * @return string
   */
  public function getLabel()
  {
    return 'Micropayment Zahlungsarten Order First';
  }

  /**
   * install
   *
   * @return array|bool
   * @throws \Enlight_Exception
   */
  public function install()
  {
    $msg       = 'Installationsfehler: <br />';
    $swVersion = Shopware()->Config()->version;
    if (
      (version_compare($swVersion, "4.3.7", ">"))
      && (version_compare($swVersion, "5.2.0", "<"))
    ) {
      if (!$this->assertRequiredPluginsPresent(array('Payment'))) {
        $msg .= "This plugin requires the plugin payment<br />";
        $this->uninstall();
        throw new Enlight_Exception("This plugin requires the shopware plugin payment");
      }
    };

    try {
      $this->createEvents();
      $msg .= '* register event handler<br />';
    } catch (Exception $e) {
      $this->logError($msg, $e);
    }
    try {
      $this->createPayments();
      $msg .= '* install payments<br />';
    } catch (Exception $e) {
      $this->logError($msg, $e);
    }
    try {
      $this->createForm();
      $msg .= '* create form<br />';
    } catch (Exception $e) {
      $this->logError($msg, $e);
    }
    try {
      $this->addSnippets();
      $msg .= '* add snippets<br />';
    } catch (Exception $e) {
      $this->logError($msg, $e);
    }

    // overwrite $msg if install was successful
    $msg = 'Installation erfolgreich: '.$this->getVersion();
    $this->Logging($msg);

    return array('success' => TRUE, 'invalidateCache' => array('frontend'));
  }

  /**
   * getCapabilities
   *
   * @return array
   */
  public function getCapabilities()
  {
    return array(
      'install' => TRUE,
      'update'  => TRUE,
      'enable'  => TRUE,
    );
  }

  /**
   * createEvents
   */
  protected function createEvents()
  {
    $this->subscribeEvent(
      'Enlight_Controller_Dispatcher_ControllerPath_Frontend_MicropaymentOF',
      'onGetControllerPathFrontend'
    );
    $this->subscribeEvent(
      'Enlight_Controller_Action_PreDispatch_Frontend_Checkout',
      'onPreDispatch'
    );
    /*
    $this->subscribeEvent(
      'Shopware_Controllers_Frontend_Checkout::confirmAction::after',
      'onConfirmAction'
    );
    */
  }

  /**
   * createPayments
   */
  protected function createPayments()
  {
    try {
      $dbErrors = '';
      $pm       = $this->getAvailablePaymethods();

      define('MCPAY_OF_BASE_PATH', dirname(__FILE__).'/mcpay/');
      require_once(MCPAY_OF_BASE_PATH.'classes/class.mcpay.php');
      $this->McPay = new mcpay();

      foreach ($pm as $k => $v) {
        $sql = 'SELECT * FROM `s_core_paymentmeans` WHERE `name`="mcpayof_'.$v['name'].'";';
        try {
          $getOldPayments = Shopware()->Db()->fetchRow($sql);
          $dbErrors       .= Shopware()->Db()->getErrorMessage();

        } catch (Exception $e) {
          if (empty($dbErrors)) {
            $this->Logging('createPayments Fetching old paymethods| '.$e->getMessage());
            return;
          } else {
            $this->Logging('createPayments Fetching old paymethods | '.$e->getMessage().' | DB-Errors: '.$dbErrors);
            return;
          }
        }

        if (!empty($getOldPayments['id'])) { // Payment already exists
          $newData = array("pluginID" => (int)$this->getId());
          $where   = array("id = ".(int)$getOldPayments['id']);
          //Shopware()->Payments()->update($newData, $where);

          try {
            $affRows  = Shopware()->Db()->update('s_core_paymentmeans', $newData, $where);
            $dbErrors .= Shopware()->Db()->getErrorMessage();
          } catch (Exception $e) {
            if (empty($dbErrors)) {
              $this->Logging('createPayments updateing old paymethods| '.$e->getMessage());
              return;
            } else {
              $this->Logging('createPayments updateing old paymethods| '.$e->getMessage().' | DB-Errors: '.$dbErrors);
              return;
            }
          }

        } else { // Payment is new
          $bind = array(
            ':name'                  => "mcpayof_".$v['name'],
            ':description'           => 'MicropaymentOF '.$v['desc'],
            ':action'                => 'MicropaymentOF',
            ':active'                => 0,
            ':pluginID'              => $this->getId(),
            ':position'              => "1".$k,
            ':additionaldescription' => $this->McPay->getPoweredBy().$this->McPay->getLogo($v['name'], $v['desc']),
          );
          $sql  = 'INSERT INTO `s_core_paymentmeans` (name, description, action, active, pluginID, position, additionaldescription)
								VALUES (:name, :description, :action, :active, :pluginID, :position, :additionaldescription);';
          try {
            Shopware()->Db()->query($sql, $bind);
            $dbErrors .= Shopware()->Db()->getErrorMessage();
          } catch (Exception $e) {
            if (empty($dbErrors)) {
              $this->Logging('createPayments inserting new paymethods| '.$e->getMessage());
              return;
            } else {
              $this->Logging('createPayments inserting new paymethods| DB-Errors: '.$dbErrors.' | '.$e->getMessage());
              return;
            }
          }

        }
      };

    } catch (Exception $e) {
      $this->Logging('createPayments | '.$e->getMessage());
      return;
    }
  }

  /**
   * createForm
   */
  protected function createForm()
  {
    try {
      $form = $this->Form();

      $form->setElement('text', 'MCPAY_OF_PROJECTID', array('label' => 'Project ID', 'value' => '', 'required' => TRUE, 'scope' => \Shopware\Models\Config\Element::SCOPE_SHOP));
      $form->setElement('text', 'MCPAY_OF_ACCESSKEY', array('label' => 'AccessKey', 'value' => '', 'required' => TRUE, 'scope' => \Shopware\Models\Config\Element::SCOPE_SHOP));

      $form->setElement('combo', 'MCPAY_OF_TESTMODE', array(
          'label' => 'Test Mode',
          'value' => 1,
          'store' => array(
            array(0, 'Nein'),
            array(1, 'Ja'),
          ),
          'scope' => \Shopware\Models\Config\Element::SCOPE_SHOP)
      );
      $form->setElement('combo', 'MCPAY_OF_DOMAIN', array(
          'label' => 'Micropayment Domain',
          'value' => 'micropayment.de',
          'store' => array(
            array('micropayment.de', 'micropayment.de'),
            array('micropayment.ch', 'micropayment.ch'),
          ),
          'scope' => \Shopware\Models\Config\Element::SCOPE_SHOP)
      );

      $form->setElement('text', 'MCPAY_OF_THEME', array('label' => 'Theme', 'value' => 'x1', 'scope' => \Shopware\Models\Config\Element::SCOPE_SHOP));
      $form->setElement('text', 'MCPAY_OF_GFX', array('label' => 'GFX', 'value' => '', 'scope' => \Shopware\Models\Config\Element::SCOPE_SHOP));
      $form->setElement('text', 'MCPAY_OF_BGCOLOR', array('label' => 'Background Color', 'value' => '#ffffff', 'scope' => \Shopware\Models\Config\Element::SCOPE_SHOP));
      $form->setElement('text', 'MCPAY_OF_BGGFX', array('label' => 'Background GFX', 'value' => '', 'scope' => \Shopware\Models\Config\Element::SCOPE_SHOP));
      $form->setElement('text', 'MCPAY_OF_SECRETFIELD', array('label' => 'Secret Field', 'value' => '', 'scope' => \Shopware\Models\Config\Element::SCOPE_SHOP));
      $form->setElement('text', 'MCPAY_OF_PAYTEXT', array('label' => 'Paytext', 'value' => 'Order: {$orderid}', 'scope' => \Shopware\Models\Config\Element::SCOPE_SHOP));
      //$form->setElement('text', 'MCPAY_OF_', array('label'=>'','value'=>'', 'scope'=>\Shopware\Models\Config\Element::SCOPE_SHOP));

      $form->setElement('combo', 'MCPAY_OF_DEBUG', array(
          'label' => 'Debugging',
          'value' => 1,
          'store' => array(
            array(0, 'Nein'),
            array(1, 'Ja'),
          ),
          'scope' => \Shopware\Models\Config\Element::SCOPE_SHOP)
      );
      $form->setElement('text', 'MCPAY_OF_DEBUG_LOG', array('label' => 'Debug Log', 'value' => '', 'scope' => \Shopware\Models\Config\Element::SCOPE_SHOP));

      //$form->save();
    } catch (Exception $e) {
      $this->Logging('createForm | '.$e->getMessage());
      return;
    }
  }

  /**
   * uninstall
   *
   * @return array|bool|void
   */
  public function uninstall()
  {
    try {
      $newData = array("active" => 0);
      $where   = array("pluginID = ".(int)$this->getId());
      //Shopware()->Payments()->update($newData, $where);

      try {
        $where = array("id = ".(int)$this->getId());
        Shopware()->Db()->update('s_core_plugins', $newData, $where);
        $dbErrors = Shopware()->Db()->getErrorMessage();
      } catch (Exception $e) {
        if (empty($dbErrors)) {
          $this->Logging('uninstall deactivate plugin | '.$e->getMessage());
          return;
        } else {
          $this->Logging('uninstall deactivate plugin '.$e->getMessage().' | DB-Errors: '.$dbErrors);
          return;
        }
      }

      return TRUE;
    } catch (Exception $e) {
      $this->Logging('uninstall failed | '.$e->getMessage());
      return;
    }
  }

  /**
   * onGetControllerPathFrontend
   *
   * @param \Enlight_Event_EventArgs $args
   *
   * @return string
   */
  public static function onGetControllerPathFrontend(Enlight_Event_EventArgs $args)
  {
    Shopware()->Template()->addTemplateDir(dirname(__FILE__).'/Views/');
    return dirname(__FILE__).'/Controllers/Frontend/MicropaymentOF.php';
  }

  /**
   * onPreDispatch
   *
   * @param \Enlight_Event_EventArgs $args
   */
  public static function onPreDispatch(Enlight_Event_EventArgs $args)
  {
    Shopware()->Template()->addTemplateDir(dirname(__FILE__).'/Views/');
    $action = $_GET['action'];

    // show prepay text on finish page
    if (in_array($action, array('finish'))){
      $view = Shopware()->Template();
      if (!empty(Shopware()->Session()->McPayChkSum)) {
        require_once(__DIR__.'/mcpay/classes/class.mcpay.php');
        $view->assign('mcpayimg',\mcpay::getPoweredBy(FALSE, Shopware()->Session()->McPayChkSum, 10));
        Shopware()->Session()->McPayChkSum = FALSE; // no reuse for chksum
      }
      // prepayment infos text
      if (!empty(Shopware()->Session()->McPayPrepayText)) {
        $view->assign('mcpay_infos', Shopware()->Session()->McPayPrepayText);
        Shopware()->Session()->McPayPrepayText = false; // no need to save this longer
      }
      // sepa info text
      if (!empty(Shopware()->Session()->McPayMandatText)){
        $view->assign('mcpay_infos', Shopware()->Session()->McPayMandatText);
        Shopware()->Session()->McPayMandatText = false; // no need to save this longer
      }
    }

    //return dirname(__FILE__).'/Controllers/Frontend/Micropayment.php';
  }

  /**
   * getAvailablePaymethods
   *
   * @return array
   */
  public function getAvailablePaymethods()
  {
    return array(
      '0' => array(
        'name' => 'ccard',
        'desc' => 'Kreditkarte',
      ),
      '1' => array(
        'name' => 'sepa',
        'desc' => 'Lastschrift',
      ),
      '2' => array(
        'name' => 'prepay',
        'desc' => 'Vorkasse',
      ),
      '3' => array(
        'name' => 'sofort',
        'desc' => 'Sofortüberweisung',
      ),
      '4' => array(
        'name' => 'paypal',
        'desc' => 'PayPal',
      ),
      '5' => array(
        'name' => 'paysafecard',
        'desc' => 'paysafecard',
      ),
      '6' => array(
        'name' => 'paydirekt',
        'desc' => 'paydirekt',
      ),
      '7' => array(
        'name' => 'call2pay',
        'desc' => 'Call2Pay',
      ),
      '8' => array(
        'name' => 'handypay',
        'desc' => 'HandyPay',
      ),
    );
  }

  /**
   * getActivePaymethods
   *
   * @return array|void
   */
  public function getActivePaymethods()
  {
    try {
      $dbErrors = '';

      $sql = 'SELECT * FROM `s_core_paymentmeans` WHERE `name` LIKE "mcpayof\_%";';
      try {
        $getOldPayments = Shopware()->Db()->fetchAssoc($sql);
        $dbErrors       .= Shopware()->Db()->getErrorMessage();

        return $getOldPayments;
      } catch (Exception $e) {
        if (empty($dbErrors)) {
          $this->Logging('createPayments Fetching old paymethods| '.$e->getMessage());
          return;
        } else {
          $this->Logging('createPayments Fetching old paymethods | '.$e->getMessage().' | DB-Errors: '.$dbErrors);
          return;
        }
      }
    } catch (Exception $e) {
      $this->Logging('getActivePaymethods | '.$e->getMessage());
      return;
    }
  }

  /**
   * createSnippets
   *
   * @return bool
   * @throws \Zend_Db_Adapter_Exception
   */
  public function createSnippets()
  {
    $snippets = $this->getSnippets();

    foreach ($snippets AS $k => $v) {

      if (!empty($v[0])) {
        $sql       = 'SELECT `id`
				        FROM `s_core_snippets`
				        WHERE `namespace` = "'.$v[0].'"
				        AND `shopID`      = 1
				        AND `localeID`    = "'.$v[1].'"
				        AND `name`        = "'.$v[2].'"
				       ';
        $data      = Shopware()->Db()->fetchAll($sql);
        $sql       = "";
        $sqlMiddle = 'SET `namespace`   = "'.$v[0].'",
                          `shopID`      = 1,
                          `localeID`    = "'.$v[1].'",
                          `name`        = "'.$v[2].'",
                          `value`       = "'.$v[3].'"
                      ';
        if ($data[0][id] > 0) {
          $sql = 'UPDATE `s_core_snippets` '.$sqlMiddle.' WHERE `id` = "'.$data[0]['id'].'"';
        } else {
          $sql = 'INSERT `s_core_snippets` '.$sqlMiddle;
        }
        Shopware()->Db()->query($sql);
      }
    }
    return TRUE;
  }

  /**
   * getSnippets
   *
   * @return array
   */
  private function getSnippets()
  {
    $snippets   = array();
    $snippets[] = array('frontend/micropaymentof/gateway', '1', 'PaymentHeader', 'Bitte f&uuml;hren Sie nun die Zahlung durch:');
    $snippets[] = array('frontend/micropaymentof/gateway', '2', 'PaymentHeader', 'Please confirm your payment:');

    $snippets[] = array('frontend/micropaymentof/gateway', '1', 'PaymentInfoWait', 'Bitte warten...');
    $snippets[] = array('frontend/micropaymentof/gateway', '2', 'PaymentInfoWait', 'Please wait ..');

    $snippets[] = array('frontend/micropaymentof/cancel', '1', 'PaymentProcess', 'Bezahlvorgang');
    $snippets[] = array('frontend/micropaymentof/cancel', '2', 'PaymentProcess', 'Payment process');

    $snippets[] = array('frontend/micropaymentof/cancel', '1', 'PaymentCancel', 'Der Bezahlvorgang wurde von Ihnen abgebrochen.');
    $snippets[] = array('frontend/micropaymentof/cancel', '2', 'PaymentCancel', 'The payment process was canceled by you.');

    $snippets[] = array('frontend/micropaymentof/cancel', '1', 'basket', 'Zur&uuml;ck zum Warenkorb');
    $snippets[] = array('frontend/micropaymentof/cancel', '2', 'basket', 'back to basket');

    $snippets[] = array('frontend/micropaymentof/error', '1', 'PaymentProcess', 'Bezahlvorgang');
    $snippets[] = array('frontend/micropaymentof/error', '2', 'PaymentProcess', 'Payment process');

    $snippets[] = array('frontend/micropaymentof/error', '1', 'PaymentError', 'Es ist ein Fehler bei Ihrem Bezahlvorgang aufgetreten. Bitte wenden Sie sich an den Shopbetreiber.');
    $snippets[] = array('frontend/micropaymentof/error', '2', 'PaymentError', 'An error occurred during your payment process. Please contact the shop owner.');

    $snippets[] = array('frontend/micropaymentof/error', '1', 'basket', 'Zur&uuml;ck zum Warenkorb');
    $snippets[] = array('frontend/micropaymentof/error', '2', 'basket', 'back to basket');

    $snippets[] = array('frontend/micropaymentof/success', '1', 'PaymentSuccess', 'Ihr Bezahlvorgang war erfolgreich!');
    $snippets[] = array('frontend/micropaymentof/success', '2', 'PaymentSuccess', 'Your transaction was successfull!');

    $snippets[] = array('frontend/micropaymentof/success', '1', 'PaymentProcess', 'Bezahlvorgang');
    $snippets[] = array('frontend/micropaymentof/success', '2', 'PaymentProcess', 'Payment process');

    return $snippets;
  }

  /**
   * logError
   *
   * @param $msg
   * @param $e
   */
  public static function logError($msg, $e)
  {
    self::Logging($msg.$e->getMessage());
  }

  /**
   * Logging
   *
   * @param $msg
   *
   * @return false|void|\Zend_Db_Statement_Pdo
   */
  public static function Logging($msg)
  {
    try {
      $sql = 'INSERT INTO `s_core_log` (`id`, `type`, `key`, `text`, `date`, `user`, `ip_address`, `user_agent`, `value4`) VALUES(?,?,?,?,?,?,?,?,?)';

      $prms_id         = NULL;
      $prms_type       = 'backend';
      $prms_key        = 'Plugin - '.self::getLabel();
      $prms_text       = $msg;
      $prms_date       = date('Y-m-d H:i:s');
      $prms_user       = self::getLabel();
      $prms_ip_address = '';
      $prms_user_agent = '';
      $prms_value4     = '';

      $params = array($prms_id, $prms_type, $prms_key, $prms_text, $prms_date, $prms_user, $prms_ip_address, $prms_user_agent, $prms_value4);

      return Shopware()->Db()->query($sql, $params);

    } catch (Exception $e) {
      self::Logging('Logging | '.$e->getMessage());
      return;
    }
  }

}
