<?php
/**
 * Micropayment GmbH
 *
 * @link       http://www.micropayment.de
 * @copyright  Copyright (c) 2020, Micropayment GmbH
 * @author     Tobias Eilers
 * @package    Shopware
 * @subpackage Plugins
 */

use Shopware\Components\CSRFWhitelistAware;

class Shopware_Controllers_Frontend_MicropaymentOF extends Shopware_Controllers_Frontend_Payment implements CSRFWhitelistAware
{
  var $McPay;
  var $cfg;

  /**
   * function to deactivate CSRF token
   * validation for specified actions
   *
   * @return array of ignored actions
   */
  public function getWhitelistedCSRFActions()
  {
    return array(
      'response',
      'success',
      'gateway',
      'notify',
    );
  }

  /**
   * Shopware_Controllers_Frontend_Micropayment constructor.
   *
   * @param \Enlight_Controller_Request_Request   $request
   * @param \Enlight_Controller_Response_Response $response
   *
   * @throws \Enlight_Event_Exception
   * @throws \Enlight_Exception
   */
  function __construct(Enlight_Controller_Request_Request $request = NULL, Enlight_Controller_Response_Response $response = NULL)
  {
    define('MCPAY_OF_BASE_PATH', dirname(__FILE__).'/../../mcpay/');
    require_once(MCPAY_OF_BASE_PATH.'classes/class.mcpay.php');
    $this->McPay = new mcpay();
    $config      = $this->Config();
    $this->cfg   = $this->getCfg($config);
    //echo '<pre>'.print_r($this->cfg,true).'</pre>';
    
    parent::__construct($request, $response);
  }

  /**
   * indexAction
   *
   * @throws \Exception
   */
  public function indexAction()
  {
    $mpBootstrap  = Shopware()->Plugins()->Frontend()->MicropaymentOF();
    $availPayment = $mpBootstrap->getAvailablePaymethods();
    $tmp          = array();
    foreach ($availPayment as $k => $v) {
      $tmp[] = $v['name'];
    }
    $activePayment = preg_replace('/mcpayof_/', '', $this->getPaymentShortName());
    //echo '<pre>'.print_r($activePayment,1).'</pre>';

    if (in_array($activePayment, $tmp, TRUE)) {
      return $this->redirect(array('controller' => 'MicropaymentOF', 'action' => 'gateway', 'forceSecure' => 1));
    } else {
      return $this->forward('index', 'checkout');
    }
  }

  /**
   * gatewayAction
   *
   * @return bool
   * @throws \Exception
   */
  public function gatewayAction()
  {
    $cfg                = $this->cfg;
    $user               = $this->getUser();
    $userData           = $this->getUserData($user, $cfg);
    $cfg['extraParams'] = array(
      'sessID' => Shopware()->Session()->sessionId,
    );

    // PaymentMethode
    $activePayment = preg_replace('/mcpayof_/', '', $this->getPaymentShortName());
    $userData['chksum']['pm'] = $activePayment;
    Shopware()->Session()->McPayChkSum = serialize($userData['chksum']);

    $ip = $_SERVER['REMOTE_ADDR'];
    if (!empty($_SERVER['HTTP_X_FORWARDED_FOR'])){ // add this to config.php -> 'trustedProxies' => ['127.0.0.1'],
      $ip = $_SERVER['HTTP_X_FORWARDED_FOR'];
    }
    $ip = Shopware()->Container()->get('shopware.components.privacy.ip_anonymizer')->anonymize((string) $ip);
    Shopware()->Session()->McPayCustomerIP = $ip;
    if ($this->McPay->debug) $this->McPay->log->debug(__CLASS__.'->'.__FUNCTION__.'->config('.$activePayment.'): '.print_r($cfg, 1).print_r($userData, 1));

    $forwardURL = '';
    switch ($activePayment) {
      /*
      case 'ccard':
        break;
      case 'sepa':
        break;
      case 'prepay':
        break;
      */
      case 'call2pay':
      case 'handypay':
        $onlyVirtual = $this->basketContainsOnlyVirtualProducts();
        if (!$onlyVirtual) {
          Shopware()->Session()->LastMPError = 'This paymethod is only available for virtual products.';
          return $this->forward('error');
        } else {
          $forwardURL = $this->McPay->getPaymentWindowParams($userData, $cfg, $activePayment);
        }
        break;
      default:
        $forwardURL = $this->McPay->getPaymentWindowParams($userData, $cfg, $activePayment);
        //echo '<pre>'.print_r($forwardURL, 1).'</pre>';
        break;
    }

    if (!empty($forwardURL)) {
      header('Location: '.$forwardURL);
    }

    exit();
    return FALSE;
  }

  /**
   * actualPaydataAction
   *
   */
  public function actualPaydataAction()
  {
    $req = $this->Request()->getParams();
    //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->$req', print_r($req,1));
    $cfg                = $this->cfg;
    $user               = $this->getUser();
    $userData           = $this->getUserData($user, $cfg);
    //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->$userData', print_r($userData,1));

    $chkSum = $userData['chksum'];
    $chkSum['pm'] = str_replace('mcpayof_', '', $req['payment']);
    $content = $this->McPay->getPoweredBy(FALSE, $chkSum, 1);
    $this->View()->assign('content', $content);
  }

  /**
   * checkUpdateAction
   * check for plugin updates at micropayment
   */
  public function checkUpdateAction()
  {
    Shopware()->Template()->addTemplateDir(dirname(__FILE__).'/Views/');

    $mpBootstrap = Shopware()->Plugins()->Frontend()->MicropaymentOF();
    $infos       = $mpBootstrap->getInfo();
    $config      = $this->cfg;
    $chksumData = array(
      'host'          => $_SERVER['HTTP_HOST'],
      'project'       => $config['project'],
      'shoptype'      => 'shopware',
      'shopversion'   => Shopware()->Config()->version,
      'plugintype'    => 'WEBOF',
      'pluginversion' => (string)$infos['version'],
      'status'        => 'stable', // check for stable plugins only (all, stable, beta and alpha)
    );
    $content = '<script src=\''.mcpay::getVersionURL().'?d='.serialize($chksumData).'\'></script>';
    $this->view->assign('content', $content);
  }

  /**
   * basketContainsOnlyVirtualProducts
   *
   * @return bool
   */
  public function basketContainsOnlyVirtualProducts()
  {
    $basket = $this->getBasket();
    //echo '<pre>'.print_r($basket,1).'</pre>';
    $onlyVirtual = TRUE;
    foreach ($basket['content'] AS $k => $v) {
      if ($v['esd'] <= 0) {
        $onlyVirtual = FALSE;
      }
    }
    return $onlyVirtual;
  }

  public function updateTempID($tempID, $orderNumber)
  {
    $sql = '
            UPDATE s_order
            SET  temporaryID = "'.$tempID.'"
            WHERE ordernumber="'.$orderNumber.'" 
        ';
    Shopware()->Db()->executeQuery($sql);
  }

  public function getOrderIDByOrdernumber($orderNr)
  {
    $db = Shopware()->Db();
    $sql = 'SELECT `id` FROM `s_order` WHERE `ordernumber` = :orderNr ';
    return $db->fetchOne($sql, ['orderNr' => $orderNr]);
  }

  /**
   * responseAction
   *
   * @throws \Exception
   */
  public function responseAction()
  {
    $crlf = "\r\n";
    if (empty($_GET['sessionId'])) {
      if (empty($_GET['sessID'])) {
        echo 'status=error'.$crlf.'message=sessionId cant be empty.';
        return;
      } else {
        $sessID = $_GET['sessID'];
      }
    } else {
      $sessID = $_GET['sessionId'];
    }

    $url     = $this->Front()->Router()->assemble(array(
      'forceSecure' => 1,
      'action'      => 'notify',
    ));
    //$url = str_replace('http://php72.docker:8080/', 'http://localhost/', $url);
    $request = $this->Request();
    $params  = $request->getParams();
    unset($params['action']);
    unset($params['controller']);
    unset($params['module']);
    $params['intresp'] = 1;
    $url               .= '?'.http_build_query($params);
    $data              = array();
    $cookies           = array(session_name() => $sessID);
    $res               = $this->McPay->doCall($url, $data, TRUE, 0, $cookies);
    echo $res;
  }

  /**
   * notifyAction
   *
   * @throws \Exception
   */
  public function notifyAction()
  {
    $request = $this->Request();
    $user    = $this->getUser();

    $return = '';
    $crlf   = "\r\n";
    $cfg    = $this->cfg;
    $post   = $_GET;

    if (!empty($post['secretfield'])) {
      $secretfield = $cfg['secretfield'];
      if ($secretfield != $post['secretfield']) {
        $return = 'status=error'.$crlf.'message=Communication is insecure.';
      }
    }

    // check mandatory params
    if (empty($post['function'])) $return = 'status=error'.$crlf.'message=Function missing.';
    else                            $func = $post['function'];

    if ($func != 'custinfo') {
      if ($this->McPay->debug) $this->McPay->log->debug(__CLASS__.'->'.__FUNCTION__.'->get'.print_r($_GET, 1));

      if (empty($post['intresp'])) $return = 'status=error'.$crlf.'message=Please use responseAction as notification url.';

      if (empty($post['orderid'])) $return = 'status=error'.$crlf.'message=OrderID missing.';
      else                          $orderId = $post['orderid'];

      if (empty($post['auth'])) $return = 'status=error'.$crlf.'message=Auth missing.';
      else                          $authId = $post['auth'];

      if (empty($post['amount'])) $return = 'status=error'.$crlf.'message=Amount missing.';
      else                          $amount = $post['amount'];

      if (empty($post['currency'])) $return = 'status=error'.$crlf.'message=Currency missing.';
      else                          $currency = $post['currency'];

      if (!isset($post['testmode'])) $return = 'status=error'.$crlf.'message=Testmode missing.';
      else                          $testmode = $post['testmode'];

      if (!empty($orderId)) {
        // extract Order ID
        if (strpos($orderId, '-') !== FALSE) {
          $parts   = explode('-', $orderId);
          $orderId = $parts[0];
          $custId  = $parts[1];
        }
        // get transactionID for normal handling below
        $order = Shopware()->Modules()->Order();
        $oID = $this->getOrderIDByOrdernumber($orderId);
        $orderData = $order->getOrderById($oID);
        $orderNumber = $orderId;
        $orderId = $orderData['transactionID'];
        $payAuth = $authId;
        $authId = 'AUTHID';

      } else {
        $return = 'status=error'.$crlf.'message=OrderID missing!';
      }
    } else {
      if (empty($post['ak'])) $return = 'status=error'.$crlf.'message=Missing KEY!';
      else                      $ak = $post['ak'];
    }

    if (!empty($return)) {
      echo $return;
      return;
    }
    // default error after pre-checks
    $return = 'status=error'.$crlf.'message=Unknown Error';

    switch ($func) {

      // status call
      case 'custinfo':
        //echo '<pre>'.print_r($cfg,1).'</pre>';
        $accessKey = $cfg['accessKey'];
        //echo $accessKey.' == '.$ak;
        if ($accessKey == $ak) {
          if (!empty($post['log'])) {
            //echo $this->McPay->readLogFile($this->McPay->log->error_log, TRUE);
            echo $this->McPay->readerLogFile($this->McPay->log->error_log);
            exit();
          } else {
            $mpBootstrap = Shopware()->Plugins()->Frontend()->MicropaymentOF();
            $infos       = $mpBootstrap->getInfo();
            echo '<pre>info:'.print_r($infos, 1).'</pre>';
            echo '<pre>CFG:'.print_r($cfg, 1).'</pre>';
            $activePayment = $mpBootstrap->getActivePaymethods();
            echo '<pre>Paymentmeans:'.print_r($activePayment, 1).'</pre>';
            $return = '';
          }
        } else {
          $return = 'status=error'.$crlf.'message=Access denied!';
        }
        break;

      // billing call
      case 'billing':
      case 'init':
        $transactionId   = $orderId;
        $paymentUniqueId = $authId;
        $sendStatusMail  = false;
        $paymentStatus   = 12; // default payment status is "12 Komplett bezahlt"
        if ($func == 'init')
          $paymentStatus   = 19; // delayed
        //$orderNumber     = $this->saveOrder($transactionId, $paymentUniqueId, $paymentStatus);
        //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->$orderNumber', print_r($orderNumber,1));
        $this->savePaymentStatus($transactionId, $paymentUniqueId, $paymentStatus, $sendStatusMail);

        $this->updateTempID($payAuth, $orderNumber);

        Shopware()->Session()->LastMcPayTrans = $payAuth;

        //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->Shopware()->Session()->McPayCustomerIP', print_r(Shopware()->Session()->McPayCustomerIP,1));
        if (!empty(Shopware()->Session()->McPayCustomerIP)) {
          $ip = Shopware()->Session()->McPayCustomerIP;
          $ip  = Shopware()->Container()->get('shopware.components.privacy.ip_anonymizer')->anonymize((string)$ip);
          $sql = 'UPDATE `s_order` SET `remote_addr` = ? WHERE `temporaryID` = ?';
          Shopware()->Db()->query($sql, array($ip, Shopware()->Session()->LastMcPayTrans));
        }

        // Add Infos to Order
        $internalcomment = '';
        $params          = array(
          'o_attr1'         => $transactionId,
          'o_attr2'         => $payAuth,
          'o_attr3'         => $amount,
          'o_attr4'         => "Title: ".$this->Request()->getParam('title'),
          'o_attr5'         => "Testmode: ".$testmode,
          'comment'         => "AuthId: ".$payAuth,
          'internalcomment' => $internalcomment,
        );
        // save paytoken
        if (!empty($post['paytoken'])){
          $params['o_attr6'] = $post['paytoken'];
        }
        if ($paymentStatus == 12) { // if payment status "Komplett bezahlt"
          $params['cleareddate'] = date('Y-m-d H:i:s');
        }
        $this->addOrderInfos($transactionId, $params);

        $url    = $this->Front()->Router()->assemble(array(
          'forceSecure' => 1,
          'action'      => 'success',
        ));
        $return = 'status=ok'.$crlf // ok  | error
          .'url='.$url.$crlf
          .'target=_top'.$crlf
          .'forward=1'.$crlf
          .'message=OK'.$crlf;
        break;

      case 'error': // error
        // back to checkout and show error
        try {
          $transactionId   = $orderId;
          $paymentUniqueId = $authId;
          $paymentStatusId = '35'; // process cancelled
          $sendStatusMail  = FALSE;
          $this->savePaymentStatus($transactionId, $paymentUniqueId, $paymentStatusId, $sendStatusMail);
        } catch (Exception $e) {
          $this->McPay->log->debug(__CLASS__.'->'.__FUNCTION__.'->error: '.$e->getMessage());
        }
        $url    = $this->Front()->Router()->assemble(array(
          'forceSecure' => 1,
          'action'      => 'checkout',
        ));
        $return = 'status=ok'.$crlf // ok  | error
          .'url='.$url.$crlf
          .'target=_top'.$crlf
          .'forward=1'.$crlf
          .'message=OK'.$crlf;
        //$return = 'status=ok'.$crlf.'message=ERROR done.'.$crlf;
        break;

      case 'payin': // payin prepayment
        try {
          $transactionId   = $orderId;
          $res             = $this->getOrderAttributes($transactionId);
          $paymentUniqueId = $res['attribute2'];
          $paymentStatusId = '11'; // payment partially
          // paystatus = PAID oder OVERPAID
          if (in_array($post['paystatus'], array('PAID', 'OVERPAID'))){
            if ($amount >= $res['attribute3']) $paymentStatusId = '12'; // payment complete
          }
          $sendStatusMail  = FALSE;
          $this->savePaymentStatus($transactionId, $paymentUniqueId, $paymentStatusId, $sendStatusMail);
        } catch (Exception $e) {
          $this->McPay->log->debug(__CLASS__.'->'.__FUNCTION__.'->payin: '.$e->getMessage());
        }
        $return = 'status=ok'.$crlf.'message=PAYIN done.'.$crlf;
        break;

      case 'expire': // expire prepayment
        try {
          $transactionId   = $orderId;
          $res             = $this->getOrderAttributes($transactionId);
          $paymentUniqueId = $res['attribute2'];
          $paymentStatusId = '35'; // payment partially
          $sendStatusMail  = FALSE;
          $this->savePaymentStatus($transactionId, $paymentUniqueId, $paymentStatusId, $sendStatusMail);
        } catch (Exception $e) {
          $this->McPay->log->debug(__CLASS__.'->'.__FUNCTION__.'->expire: '.$e->getMessage());
        }
        $return = 'status=ok'.$crlf.'message=EXPIRE done.'.$crlf;
        break;

      case 'storno': // chargeback
        try {
          $transactionId   = $orderId;
          $res             = $this->getOrderAttributes($transactionId);
          $paymentUniqueId = $res['attribute2'];
          $paymentStatusId = '35'; // cancelled
          $sendStatusMail  = FALSE;
          $this->savePaymentStatus($transactionId, $paymentUniqueId, $paymentStatusId, $sendStatusMail);
        } catch (Exception $e) {
          $this->McPay->log->debug(__CLASS__.'->'.__FUNCTION__.'->storno: '.$e->getMessage());
        }
        $return = 'status=ok'.$crlf.'message=STORNO done.'.$crlf;
        break;

      case 'backpay': // chargeback reversal
        try {
          $transactionId   = $orderId;
          $res             = $this->getOrderAttributes($transactionId);
          $paymentUniqueId = $res['attribute2'];
          $paymentStatusId = '21'; // clarification
          $sendStatusMail  = FALSE;
          $this->savePaymentStatus($transactionId, $paymentUniqueId, $paymentStatusId, $sendStatusMail);
        } catch (Exception $e) {
          $this->McPay->log->debug(__CLASS__.'->'.__FUNCTION__.'->backpay: '.$e->getMessage());
        }
        $return = 'status=ok'.$crlf.'message=BACKPAY done.'.$crlf;
        break;

      case 'refund': // refund
        try {
          // if paystatus == PAID or OVERPAID do nothing because prepayment
          if (in_array($post['paystatus'], array('PAID', 'OVERPAID'))){
            break;
          }
          $transactionId   = $orderId;
          $res             = $this->getOrderAttributes($transactionId);
          $paymentUniqueId = $res['attribute2'];
          $paymentStatusId = '20'; // recredit
          $sendStatusMail  = FALSE;
          $this->savePaymentStatus($transactionId, $paymentUniqueId, $paymentStatusId, $sendStatusMail);
        } catch (Exception $e) {
          $this->McPay->log->debug(__CLASS__.'->'.__FUNCTION__.'->refund: '.$e->getMessage());
        }
        $return = 'status=ok'.$crlf.'message=Refund done.'.$crlf;
        break;

      case 'quit': // reversal SEPA only
        try {
          $transactionId   = $orderId;
          $res             = $this->getOrderAttributes($transactionId);
          $paymentUniqueId = $res['attribute2'];
          $paymentStatusId = '35'; // process cancelled
          $sendStatusMail  = FALSE;
          $this->savePaymentStatus($transactionId, $paymentUniqueId, $paymentStatusId, $sendStatusMail);
        } catch (Exception $e) {
          $this->McPay->log->debug(__CLASS__.'->'.__FUNCTION__.'->quit: '.$e->getMessage());
        }
        $return = 'status=ok'.$crlf.'message=QUIT done.'.$crlf;
        break;
    }

    echo $return;
    return;
  }

  /**
   * cancelAction
   */
  public function cancelAction()
  {
  }

  /**
   * successAction
   *
   * @throws \Exception
   */
  public function successAction()
  {
    // update customer IP address in DB
    /*
    $ip = $_SERVER['REMOTE_ADDR'];
    if (!empty($_SERVER['HTTP_X_FORWARDED_FOR'])){ // add this to config.php -> 'trustedProxies' => ['127.0.0.1'],
      $ip = $_SERVER['HTTP_X_FORWARDED_FOR'];
    }
    $ip = Shopware()->Container()->get('shopware.components.privacy.ip_anonymizer')->anonymize((string) $ip);
    $sql = 'UPDATE `s_order` SET `remote_addr` = ? WHERE `temporaryID` = ?';
    Shopware()->Db()->query($sql, array($ip, Shopware()->Session()->LastMcPayTrans));
    */

    return $this->redirect(array(
        'controller'  => 'checkout',
        'action'      => 'finish',
        'forceSecure' => 1,
        'sUniqueID'   => Shopware()->Session()->LastMcPayTrans,
      )
    );
    unset(Shopware()->Session()->LastMPTrans);
  }

  /**
   * errorAction
   */
  public function errorAction()
  {
    Shopware()->Template()->addTemplateDir(dirname(__FILE__).'/Views/');
    $content                    = $this->McPay->getPoweredBy(FALSE, Shopware()->Session()->McPayChkSum, -1, Shopware()->Session()->LastMPError);
    $content                    .= htmlentities(Shopware()->Session()->LastMPError);
    $this->View()->ErrorMessage = $content;
  }

  /**
   * formatNumber
   *
   * @param $value
   *
   * @return string
   */
  public function formatNumber($value)
  {
    return sprintf('%1.2f', $value);
  }

  /**
   * Config
   *
   * @return mixed
   */
  public function Config()
  {
    return Shopware()->Plugins()->Frontend()->MicropaymentOF()->Config();
  }

  /**
   * readComment
   *
   * @param $transactionID
   *
   * @return mixed
   */
  function readComment($transactionID)
  {
    $data = Shopware()->Db()->fetchAll("SELECT `comment` FROM `s_order` WHERE `transactionID` = '".$transactionID."' ");
    return $data[0]['comment'];
  }

  /**
   * createSecretHash
   *
   * @param $orderID
   *
   * @return string
   */
  function createSecretHash($orderID)
  {
    return sha1($orderID.$this->Config()->MCPAY_OF_SECRET);
  }

  /**
   * getSnippet
   *
   * @param        $name
   * @param        $localeId
   * @param string $ns
   * @param int    $shopId
   *
   * @return mixed
   */
  public function getSnippet($name, $localeId, $ns = 'frontend/micropaymentof/success', $shopId = 1)
  {
    $sql  = 'SELECT `value`
            FROM `s_core_snippets`
            WHERE `namespace` = "'.$ns.'"
            AND `shopID`      = "'.$shopId.'"
            AND `localeID`    = "'.$localeId.'"
            AND `name`        = "'.$name.'" ';
    $data = current(Shopware()->Db()->fetchAll($sql));
    return $data['value'];
  }

  /**
   * getOrder
   *
   * @param $transactionId
   *
   * @return mixed
   */
  public function getOrder($transactionId)
  {
    $sql  = 'SELECT *
            FROM `s_order`
            WHERE `transactionID` = "'.$transactionId.'"
            ';
    $data = current(Shopware()->Db()->fetchAll($sql));
    return $data;
  }

  /**
   * addOrderInfos
   *
   * @param $transactionID
   * @param $params
   *
   * @return bool|void
   */
  function addOrderInfos($transactionID, $params)
  {
    try {
      $orderModel = Shopware()->Models()->getRepository('Shopware\Models\Order\Order')->findOneBy(array('transactionId' => $transactionID));
      if (empty($orderModel)) {
        return FALSE;
      }
      $oldValue = $orderModel->getInternalComment();
      if (!empty($params['internalcomment'])) {
        $params['internalcomment'] = date('d.m.Y H:i:s')."\n".$params['internalcomment']."\n \n".$oldValue;
      } else {
        $params['internalcomment'] = $oldValue;
      }
      // Mapping database -> model
      $orderMappings = array(
        'ordernumber'          => 'number',
        'userID'               => 'customerId',
        'invoice_amount'       => 'invoiceAmount',
        'invoice_amount_net'   => 'invoiceAmountNet',
        'invoice_shipping'     => 'invoiceShipping',
        'invoice_shipping_net' => 'invoiceShippingNet',
        'ordertime'            => 'orderTime',
        'status'               => 'status',
        'cleared'              => 'cleared',
        'paymentID'            => 'paymentId',
        'transactionID'        => 'transactionId',
        'comment'              => 'comment',
        'customercomment'      => 'customerComment',
        'internalcomment'      => 'internalComment',
        'net'                  => 'net',
        'taxfree'              => 'taxFree',
        'partnerID'            => 'partnerId',
        'temporaryID'          => 'temporaryId',
        'referer'              => 'referer',
        'cleareddate'          => 'clearedDate',
        'trackingcode'         => 'trackingCode',
        'language'             => 'languageIso',
        'dispatchID'           => 'dispatch',
        'currency'             => 'currency',
        'currencyFactor'       => 'currencyFactor',
        'subshopID'            => 'shopId',
        'remote_addr'          => 'remoteAddress',
      );

      $attributeMapping = array(
        'o_attr1' => 'attribute1',
        'o_attr2' => 'attribute2',
        'o_attr3' => 'attribute3',
        'o_attr4' => 'attribute4',
        'o_attr5' => 'attribute5',
        'o_attr6' => 'attribute6');

      $newData           = array();
      $attribute         = array();
      $params['o_attr6'] = "MCPAY";
      //order mapping
      foreach ($orderMappings AS $k => $v) {
        if (isset($params[$k])) $newData[$v] = $params[$k];
      }
      //attribute mapping
      foreach ($attributeMapping as $k => $v) {
        if (isset($params[$k])) $attribute[$v] = $params[$k];
      }
      if (!empty($attribute)) {
        $newData['attribute'] = $attribute;
      }

      if (isset($params['cleared'])) {
        $sql = 'UPDATE `s_order` SET `cleared` = ? WHERE `transactionID` = ?';
        Shopware()->Db()->query($sql, array((int)$params['cleared'], $transactionID));
      }

      // populate Model with data
      $orderModel->fromArray($newData);
      Shopware()->Models()->persist($orderModel);
      // save to database
      Shopware()->Models()->flush();
    } catch (Exception $e) {
      $mpBootstrap = Shopware()->Plugins()->Frontend()->MicropaymentOF();
      $mpBootstrap->Logging('getActivePaymethods | '.$e->getMessage());
      return;
    }
  }

  /**
   * getOrderAttributes
   *
   * @param $transactionID
   *
   * @return bool|object|\Shopware\Models\Order\Order|void|null
   */
  function getOrderAttributes($transactionID)
  {
    try {
      $sql = 'SELECT * FROM `s_order_attributes` WHERE `attribute1` = ?';
      $dat = Shopware()->Db()->fetchAssoc($sql, $transactionID);
      return current($dat);
    } catch (Exception $e) {
      $mpBootstrap = Shopware()->Plugins()->Frontend()->MicropaymentOF();
      $mpBootstrap->Logging('getActivePaymethods | '.$e->getMessage());
      return;
    }
  }

  /**
   * getCfg
   *
   * @param $confObj
   *
   * @return array
   */
  function getCfg($confObj)
  {
    $configMap = array(
      'MCPAY_OF_ACCESSKEY'   => 'accessKey',
      'MCPAY_OF_BGCOLOR'     => 'bgcolor',
      'MCPAY_OF_BGGFX'       => 'bggfx',
      'MCPAY_OF_CCARDFORMID' => '',
      'MCPAY_OF_DEBUG'       => 'debug',
      'MCPAY_OF_DEBUG_LOG'   => '',
      'MCPAY_OF_DOMAIN'      => 'suffix',
      'MCPAY_OF_GFX'         => 'gfx',
      'MCPAY_OF_PAYFORMID'   => '',
      'MCPAY_OF_PROJECTID'   => 'project',
      'MCPAY_OF_SECRETFIELD' => 'secretfield',
      'MCPAY_OF_SEPAFORMID'  => '',
      'MCPAY_OF_TESTMODE'    => 'testMode',
      'MCPAY_OF_THEME'       => 'theme',
      'MCPAY_OF_PAYTEXT'     => 'paytext',
    );
    $cfg       = array();
    foreach ($configMap AS $k => $v) {
      if (empty($v)) continue;
      $cfg[$v] = $confObj->$k;
    }
    //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->$cfg', print_r($cfg,1));
    $this->McPay->setConfig($cfg);

    return $cfg;
  }

  /**
   * getUserData
   *
   * @param $user
   * @param $config
   *
   * @return array
   */
  function getUserData($user, $config)
  {
    //echo '<pre>'.print_r($user,1).'</pre>';
    //echo '<pre>'.print_r($config,1).'</pre>';

    $countryISO = Shopware()->Db()->fetchOne("
          SELECT `countryiso` FROM `s_core_countries`
          WHERE `id` = ?
          ", array($user['billingaddress']['countryID']));

    $custID  = $user['additional']['user']['userID'];
    $orderId = $this->createPaymentUniqueId();
    //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->$orderId', print_r($orderId,1));
    if (empty($custID)) { // guest order
      $custID = 'guest-'.$orderId;
    } else {
      $orderId .= '-'.$custID;
    }
    $custID = $this->McPay->unifyID($custID);

    // use real ordernumber
    $transactionId = $this->createPaymentUniqueId();
    $paymentUniqueId = 'AUTHID';
    $paymentStatus = 17; // open
    //$paymentStatus = 19; // delayed
    $orderId = $this->saveOrder($transactionId, $paymentUniqueId, $paymentStatus);

    $amount   = $this->formatNumber($this->getAmount());
    //$currency = Shopware()->Currency()->getShortName();
    $currency = $this->getCurrencyShortName();

    $mpBootstrap = Shopware()->Plugins()->Frontend()->MicropaymentOF();
    $infos       = $mpBootstrap->getInfo();
    $chksumData = array(
      'amount'        => $amount * 100,
      'currency'      => $currency,
      'iso'           => $countryISO,
      'host'          => $_SERVER['HTTP_HOST'],
      'project'       => $config['project'],
      'shoptype'      => 'shopware',
      'shopversion'   => Shopware()->Config()->version,
      'plugintype'    => 'WEB',
      'pluginversion' => (string)$infos['version'],
    );

    $shopContext = $this->get('shopware_storefront.context_service')->getShopContext();
    $lang = substr($shopContext->getShop()->getLocale()->getLocale(),0,2);

    $userData = array(
      'company'    => $user['billingaddress']['company'],
      'firstName'  => $user['billingaddress']['firstname'],
      'lastName'   => $user['billingaddress']['lastname'],
      'salutation' => 'MR',
      'street'     => $user['billingaddress']['street'].$user['shippingaddress']['streetnumber'],
      'zip'        => $user['billingaddress']['zipcode'],
      'city'       => $user['billingaddress']['city'],
      'country'    => $countryISO,
      'email'      => $user['additional']['user']['email'],
      'ip'         => $_SERVER['REMOTE_ADDR'],
      'id'         => $custID,
      'order_id'   => $orderId,
      'lang'       => $lang,
      'amount'     => ($amount * 100),
      'currency'   => $currency,
      'chksum'     => $chksumData, // request security
    );
    if (empty($userData['ip'])) $userData['ip'] = $_SERVER['REMOTE_ADDR']; // Falls IP Leer, dann aus dem Server holen



    return $userData;
  }

}